#include <u.h>
#include <libc.h>
#include <draw.h>
#include <event.h>
#include <html.h>
#include <plumb.h>
#include <keyboard.h>
#include "abaco.h"

Image *bground, *dbuf;
Rectangle viewr;
Rune status[512];

Rune location[512];
int plumbfd;
int refresh = -1;
vlong refnsec;
Docinfo *docinfo;
Item *docitems;
Rune *Url = nil;

Point ul;
Rectangle ulrange;
Point focus;

void redraw(Image *);

enum mids {
	Plumb,
	Snarf,
	Download,
	Exit,
};

char *mnames[] = {
	"plumb",
	"snarf",
	"download",
	"exit",
	nil
};

Menu mmenu = {
	mnames
};

void
drawbars(void)
{

	Rectangle r;
	Image *tim;

	r = screen->clipr;
	r.max.y = r.min.y + font->height;

	tim = allocimage(display, Rect(0, 0, Dx(screen->r), font->height), screen->chan, 0, DBlue);
	draw(tim, Rect(0, 0, Dx(r), font->height), bground, nil, ZP);
	runestring(tim, ZP, display->black, ZP, font, location);
	draw(screen, r, tim, nil, ZP);

	r = screen->clipr;
	r.min.y = r.max.y - font->height;
	draw(tim, Rect(0, 0, Dx(r), font->height), bground, nil, ZP);
	runestring(tim, ZP, display->black, ZP, font, status);
	draw(screen, r, tim, nil, ZP);
	freeimage(tim);
}

/* clip p to be in r */
Point
pclip(Point p, Rectangle r)
{
	if(p.x < r.min.x)
		p.x = r.min.x;
	else if(p.x >= r.max.x)
		p.x = r.max.x-1;

	if(p.y < r.min.y)
		p.y = r.min.y;
	else if(p.y >= r.max.y)
		p.y = r.max.y-1;

	return p;
}

/* Translate the image in the window by delta. */
void
translate(Point delta, int flag)
{
	Point u;
	int i;

	u = pclip(addpt(ul, delta), ulrange);
	delta = subpt(u, ul);
	if(delta.x==0 && delta.y==0)
		return;

	i = u.y+Dy(dbuf->r);
	if(viewr.min.y < i && i <viewr.max.y && flag)
		return;
	if(viewr.min.y < u.y && u.y < viewr.max.y && flag)
		return;

	i = u.x+Dx(dbuf->r);
	if(viewr.min.x < i && i < viewr.max.x & flag)
		return;
	if(viewr.min.x < u.x && u.x < viewr.max.x & flag)
		return;

	ul = u;

	redraw(screen);
}

Point
getpt(Point p)
{
	p = subpt(ul, p);
	p.x = abs(p.x);
	p.y = abs(p.y);

	return p;
}

void
main(int argc, char **argv)
{
	Event e;
	Rune *tmp;
	Rune *snarf = nil;
	Mouse m;
	Point oxy, xy, dxy, seltop, selbot;
	char *p;
	int refp[2];
	int i, snarfing = 0;

	ARGBEGIN{
	case 'w':
		newwin();
		break;
	}ARGEND

	if(argc != 1)
		sysfatal("usage: abaco [-w] url");

	dup(open("/tmp/abaco", OWRITE), 2);

	erfork(RFNAMEG);

	if (p = strstr(argv[0], "://"))
		p += 3;
	else p = argv[0];
	if (strchr(p, '/'))
		Url = runesmprint("%s", argv[0]);
	else Url = runesmprint("%s/", argv[0]);

	runesnprint(location, nelem(location), "%S - %S", Url,
		docinfo?docinfo->doctitle : nil);

	if(initdraw(nil, nil, "abaco") < 0)
		sysfatal("initdraw failed: %r");

	dbuf = display->white;
	plumbfd = plumbopen("send", OWRITE);
	bground = allocimagemix(display, DPalebluegreen, DWhite);
	display->locking = 1;

	unlockdisplay(display);
	pipe(refp);
	refresh = refp[1];
	einit(Emouse|Ekeyboard);
	eresized(1);
	ul = viewr.min;
	loadpage(Url);

	estart(Erefresh, refp[0], 1);
	for(;;){
		switch(event(&e)) {
		case Erefresh:
			runesnprint(location, nelem(location), "%S - %S", Url,
				docinfo?docinfo->doctitle : nil);
			if (e.data[0] == 'f')
				eresized(1);
			else if (nsec() - refnsec > 1000*1000*1000)
				eresized(1);
			break;
		case Ekeyboard:
			if(!dbuf)
				break;;
			if(kbdevent(focus, e.kbdc))
				redraw(screen);
			else{
				switch (e.kbdc){
				case 'q':
				case 0x7f:
					postnote(PNGROUP, getpid(), "die");	
					exits(0);
				case Kdown:
				Scrolldown:
					i = -Dy(viewr)/3;
					if(i < -10)
						i += 10;
					if(i+ul.y+Dy(dbuf->r) <= viewr.max.y)
						i = viewr.max.y - Dy(dbuf->r) - ul.y;
					translate(Pt(0, i), 0);
					break;
				case Kpgdown:
					i = -Dy(viewr);
					if(i < -10)
						i += 10;
					if(i+ul.y+Dy(dbuf->r) <= viewr.max.y)
						i = viewr.max.y - Dy(dbuf->r) - ul.y;
					translate(Pt(0, i), 0);
					break;
				case Kup:
				Scrollup:
					i = Dy(viewr)/3;
					if(i > 10)
						i -= 10;
					if(i+ul.y > viewr.min.y)
						i = viewr.min.y - ul.y;
					translate(Pt(0, i), 0);
					break;
				case Kpgup:
					i = Dy(viewr);
					if(i > 10)
						i -= 10;
					if(i+ul.y > viewr.min.y)
						i = viewr.min.y - ul.y;
					translate(Pt(0, i), 0);
					break;
				case Kleft:
					i = -Dx(viewr)/3;
					if(i < -10)
						i += 10;
					if(i+ul.x+Dx(dbuf->r) <= viewr.max.x)
						i = viewr.max.x - Dx(dbuf->r) - ul.x;
					translate(Pt(i, 0), 0);
					break;
				case Kright:
					i = Dy(viewr)/3;
					if(i > 10)
						i -= 10;
					if(i+ul.x > viewr.min.x)
						i = viewr.min.x - ul.x;
					translate(Pt(i, 0), 0);
					break;
				case Khome:
					ul.y = viewr.min.y;
					redraw(screen);
					break;
				case Kend:
					ul.y = viewr.max.y-Dy(dbuf->r);
					redraw(screen);
					break;
				}
			}
			break;
		case Emouse:
			if(!dbuf)
				break;
			xy = e.mouse.xy;
			if(e.mouse.buttons & 1){
				focus = getpt(xy);	/* click to focus.. */
				if(mouseevent(focus))
					redraw(screen);
				else{
					m = e.mouse;
					oxy = m.xy;
					do{
						dxy = subpt(m.xy, oxy);
						dxy.y *= 5;
						oxy = m.xy;
						translate(dxy, 1);
						m = emouse();
					}while(m.buttons & 1);
					if(m.buttons){
						dxy = subpt(xy, oxy);
						translate(dxy, 1);
					}
				break;
				}
			}else if(e.mouse.buttons & 2){
				switch(emenuhit(2, &e.mouse, &mmenu)){
				case Plumb:
					if(snarf){
						plumbrunestring(snarf);
						free(snarf);
					}else{
						if(tmp = findhref(getpt(xy))){
							tmp = geturl(docinfo->base, tmp);
							plumbrunestring(tmp);
							free(tmp);
						}
					}
					break;
				case Snarf:
					snarf = dosnarf(getpt(seltop), getpt(selbot));
					if(snarf){
						snarfrunestring(snarf);
						free(snarf);
					}else{
						if(tmp = findhref(getpt(xy))){
							tmp = geturl(docinfo->base, tmp);
							snarfrunestring(tmp);
							free(tmp);
						}
					}
					break;
				case Download:
					if(tmp=findhref(getpt(xy))){
						tmp = geturl(docinfo->base, tmp);
						download(tmp);
						free(tmp);
					}
					break;
				case Exit:
					exits(0);
					break;
				}
			}else if(e.mouse.buttons & 4){
				if(snarfing){
					redraw(screen);
					snarfing = 0;
					break;
				}
				m = e.mouse;
				selbot = m.xy;
				seltop = m.xy;
				oxy = m.xy;
				do{
					m = emouse();
					snarfing = 1;
					if(oxy.x * oxy.y < m.xy.x * m.xy.y)
						seltop = oxy, selbot = m.xy;
					else
						seltop = m.xy, selbot = oxy;
					redraw(screen);
					drawsnarf(screen, getpt(seltop), getpt(selbot), ul);
				}while(m.buttons & 4);
			}else if(e.mouse.buttons & 8){
				goto Scrollup;
			}else if(e.mouse.buttons & 16){
				goto Scrolldown;
			}else {
				if(!dbuf)
					break;
				if(tmp = findhref(getpt(xy))) {
					tmp = geturl(docinfo->base, tmp);
					if (runestrncmp(status, tmp, nelem(status))) {
						runestrncpy(status, tmp, nelem(status));
						lockdisplay(display);
						drawbars();
						unlockdisplay(display);
					}
					free(tmp);
				} else {
					status[0] = 0;
					lockdisplay(display);
					drawbars();
					unlockdisplay(display);
				}
			}
			xy = e.mouse.xy;
			break;
		}
	}
}

void
redraw(Image *screen)
{
	if(dbuf == nil)
		return;

	ulrange.max = viewr.max;
	ulrange.min = subpt(viewr.min, Pt(Dx(dbuf->r), Dy(dbuf->r)));
	ul = pclip(ul, ulrange);
	draw(screen, viewr, dbuf, nil, subpt(dbuf->r.min, subpt(ul, viewr.min)));
	flushimage(display, 0);	
}

void
eresized(int new)
{
	Rectangle r;
	r = viewr;

	lockdisplay(display);
	if(new){
		if(getwindow(display, Refnone) < 0)
			sysfatal("can't reattach to window");
		if(dbuf && dbuf != display->white)
			freeimage(dbuf);

		viewr = screen->r;
		viewr.min.y += font->height;
		viewr.max.y -= font->height;
		dbuf = render(viewr, docinfo, docitems);
		if(dbuf == nil){
			unlockdisplay(display);
			return;
		}
	}
	ul = addpt(ul, subpt(viewr.min, r.min));
	redraw(screen);
	drawbars();
	refnsec = nsec();
	unlockdisplay(display);
}
