#include <u.h>
#include <libc.h>
#include <draw.h>
#include <html.h>
#include "abaco.h"

Image *
eallocimage(Display *d, Rectangle r, ulong chan, int repl, int col)
{
	Image *i;

	i = allocimage(d, r, chan, repl, col);
	if(i == nil){
		fprint(2, "r.max.x = %d, r.max.y = %d\n", r.max.x, r.max.y);
		sysfatal("%r");
	}
	return i;
}

void
rect3d(Image *im, Rectangle r, int i, Image **c, Point sp)
{
	Point p[6];

	if(i < 0){
		r = insetrect(r, i);
		sp = addpt(sp, Pt(i,i));
		i = -i;
	}
	draw(im, Rect(r.min.x+i, r.min.y+i, r.max.x-i, r.max.y-i), c[2], nil, sp);
	p[0] = r.min;
	p[1] = Pt(r.min.x, r.max.y);
	p[2] = Pt(r.min.x+i, r.max.y-i);
	p[3] = Pt(r.min.x+i, r.min.y+i);
	p[4] = Pt(r.max.x-i, r.min.y+i);
	p[5] = Pt(r.max.x, r.min.y);
	fillpoly(im, p, 6, 0, c[0], sp);
	p[0] = r.max;
	p[1] = Pt(r.min.x, r.max.y);
	p[2] = Pt(r.min.x+i, r.max.y-i);
	p[3] = Pt(r.max.x-i, r.max.y-i);
	p[4] = Pt(r.max.x-i, r.min.y+i);
	p[5] = Pt(r.max.x, r.min.y);
	fillpoly(im, p, 6, 0, c[1], sp);
}

void
ellipse3d(Image *im, Point p, int rad, int i, Image **c, Point sp)
{
	fillarc(im, p, rad, rad, c[0], sp, 45, 180);
	fillarc(im, p, rad, rad, c[1], sp,  45, -180);
	fillellipse(im, p, rad-i, rad-i, c[2], sp);
}

static
void
colarray(Image **c, Image *c0, Image *c1, Image *c2, int checked)
{
	if(checked){
		c[0] = c0;
		c[1] = c1;
	}else{
		c[0] = c1;
		c[1] = c0;
	}
	c[2] = c2;
}

static char *fontpaths[] = {
	"/lib/font/bit/lucidasans/unicode.6.font",
	"/lib/font/bit/lucidasans/unicode.7.font",
	"/lib/font/bit/lucidasans/unicode.8.font",
	"/lib/font/bit/lucidasans/unicode.10.font",
	"/lib/font/bit/lucidasans/unicode.13.font",
	"/lib/font/bit/lucidasans/italicunicode.6.font",
	"/lib/font/bit/lucidasans/italicunicode.7.font",
	"/lib/font/bit/lucidasans/italicunicode.8.font",
	"/lib/font/bit/lucidasans/italicunicode.10.font",
	"/lib/font/bit/lucidasans/italicunicode.13.font",
	"/lib/font/bit/lucidasans/boldunicode.6.font",
	"/lib/font/bit/lucidasans/boldunicode.7.font",
	"/lib/font/bit/lucidasans/boldunicode.8.font",
	"/lib/font/bit/lucidasans/boldunicode.10.font",
	"/lib/font/bit/lucidasans/boldunicode.13.font",
	"/lib/font/bit/lucidasans/typeunicode.6.font",
	"/lib/font/bit/lucidasans/typeunicode.7.font",
	"/lib/font/bit/lucidasans/typeunicode.9.font",
	"/lib/font/bit/lucidasans/typeunicode.12.font",
	"/lib/font/bit/lucidasans/typeunicode.16.font",
};

Font *fonts[NumFnt];

Font *
getfont(int i)
{
	if(i >= NumFnt || i < 0){
		fprint(2, "bad font type %d: %r\n", i);
		exits("bad font");
	}
	if(fonts[i] == nil){
		fonts[i] = openfont(display, fontpaths[i]);
		if(fonts[i] == nil)
			sysfatal("can't open font file %s: %r\n", fontpaths[i]);
	}
	return fonts[i];
}

void
freefonts(void)
{
	int i;

	for(i=0; i<NumFnt; i++){
		if(fonts[i])
			freefont(fonts[i]);
	}
}

typedef struct Color Color;

struct Color
{
	int rgb;
	Image *i;
	Color *next;
};

static Color *colorlist;

static Color *
alloccolor(int rgb)
{
	Color *c;

	c = emalloc(sizeof(Color));
	c->i = eallocimage(display, Rect(0,0,1,1), screen->chan, 1, (rgb<<8)|0xFF);
	c->next = colorlist;
	colorlist = c;

	return c;
}

Image *
getcolor(int rgb)
{
	Color *c;

	switch(rgb){
	case 0xFFFFFF:
		return display->white;
		break;
	case 0x000000:
		return display->black;
		break;
	}

	if(colorlist == nil){
		colorlist = alloccolor(rgb);
		return colorlist->i;
	}

	for(c=colorlist; c!=nil; c=c->next)
		if(c->rgb == rgb)
			return c->i;

	c = alloccolor(rgb);

	return c->i;
}

void
freecolors(void)
{
	Color *c, *t;

	for(c=colorlist; c!=nil; c = t){
		t = c->next;
		freeimage(c->i);
		free(c);
	}
}

void
drawstringn(Image *b, Point p, Image *c, Font *f, Rune *s, int n, int ul)
{
	int y, w;

	runestringn(b, p, c, ZP, f, s, n);
	if(ul == ULnone)
		return;

	if(ul == ULmid)
		y = f->height/2;
	else
		y =f->height-1;

	w = runestringnwidth(f, s, n);
	line(b, addpt(p, Pt(0,y)), addpt(p,Pt(w,y)), Enddisc, Enddisc, 0, c, ZP);
}

enum
{
	Light = 0xEEEEEE,
	Dark = 0x666666,
	Red = 0xBB0000,
	Back = 0xAAAAAA,
};

void
drawcheck(Image *b, Rectangle r, int isradio, int checked)
{
	Image *c[3];
	int n;

	if(checked)
		colarray(c, getcolor(Dark), getcolor(Light), getcolor(Red), 1);
	else
		colarray(c, getcolor(Dark), getcolor(Light), getcolor(Back), 0);

	if(isradio){
		n = Checkwidth/2-1;
		ellipse3d(b, addpt(r.min, Pt(n,n)), n, Border, c, ZP);
	}else
		rect3d(b, r, Border, c, ZP);
}

void
drawbutton(Image *b, Rectangle r, Rune *s, int down)
{
	Point p;
	Image *c[3] = nil;

	colarray(c, getcolor(Dark), getcolor(Light), getcolor(Back), down);
	rect3d(b, r, Border, c, ZP);
	p = addpt(r.min, Pt(Butmargin, Butmargin));
	runestring(b, p, display->black, ZP, getfont(WFont), s);
}

void
drawtextarea(Image *b, Rectangle r, Rune *s)
{
	Point p;
	Image *c[3] = nil;
	Font *f;

	f = getfont(WFont);
	colarray(c, getcolor(Dark), getcolor(Light), display->white, 1);
	rect3d(b, r, Border, c, ZP);
	p = addpt(r.min, Pt(Butmargin, Butmargin));
	runestring(b, p, display->black, ZP, f, s);
}
