#include <u.h>
#include <libc.h>
#include <draw.h>
#include <html.h>
#include "abaco.h"

static int anchalloc;
static int nanch;
static Anchor **anch;

int
anchcmp(void *A, void *B)
{
	Anchor **a, **b;
	a = A; b = B;
	return (*a)->index - (*b)->index;
}


void
addanchor(Anchor *a)
{
	if (nanch == anchalloc) {
		if (anchalloc == 0) anchalloc = 32;
		anchalloc *= 2;
		anch = erealloc(anch, sizeof(Anchor *) * anchalloc);
	}
	anch[nanch++] = a;
}

void
sortanchors(void)
{
	qsort(anch, nanch, sizeof(Anchor *), anchcmp);
}


Anchor *
anchor(int i)
{
	int l, h, m;
	l = 0; h = nanch - 1;
	while (l < h) {
		m = (l + h) / 2;
		if (anch[m]->index > i)
			h = m - 1;
		else if (anch[m]->index < i)
			l = m + 1;
		else break;
	}
	m = (l + h) / 2;
	if (anch[m]->index != i) {
		fprint(2, "index %d not found\n", i);
		return nil;
	}
	return anch[m];
}

int
charset(uchar *s)
{
	char *meta, *emeta, *charset;

	meta = cistrstr((char*)s, "<meta");
	if(meta == nil)
		return ISO_8859_1;
	for(emeta=meta; *emeta!='>' && *emeta!='\0'; emeta++)
		;
	charset = strstr((char*)s, "charset=");
	if(charset == nil)
		return ISO_8859_1;
	
	charset += 8;
	if(*charset == '"')
		charset++;
	if(cistrncmp(charset, "utf-8", 5)==0 || cistrncmp(charset, "utf8", 4)==0)
		return UTF_8;
	return ISO_8859_1;
}

static
Rune *
dotdot(Rune *url)
{
	Rune *r, *p;
	while (r = runestrstr(url, L"/../")) {
		p = r  - 1;
		r += 3;
		while (p > url && *p != L'/')
			p--;
		while (*p++ = *r++)
			;
	}
	while (r = runestrstr(url, L"/./")) {
		p = r ;
		r += 2;
		while (*p++ = *r++)
			;
	}
	return url;
}

static
Rune *
javascript(Rune *url)
{
	Rune *r, *p;
	if (p = runestrstr(url, L"javascript:popup")) {
		r = p;
		while (*p && *p != L'(')
			p++;
		while (*p && *p != L'\'')
			p++;
		p++;
		while (*r++ = *p++)
			if (*p == L'\'')
				break;
		*r = 0;
	}
	return url;
}

Rune
*urlencode(Rune *s)
{
	int i;
	for (i = 0; s[i]; ++i)
		switch(s[i]) {
		case ' ':
			s[i] = '+';
			break;
		case '\t':
			s[i] = '+';
			break;
		default:
			break;
		}
	return s;
}

Rune *
urlcombine(Rune *b, Rune *u)
{
	int len;
	Rune *p, *q;

	if(runestrstr(u, L"://"))
		return runestrdup(u);

	if(*u == L'/'){
		if((p = runestrstr(b, L"://")) == nil){
			fprint(2, "'%S' garbled url\n", b);
			return runestrdup(L"");
		}
		if ((q = runestrchr(p+3, L'/')) != nil)
			return runesmprint("%.*S/%S", (int)(q-b), b, u+1);
	}


	if ((len = runestrlen(b)) < 1)
		return b;
	if (b[len -1] == L'/')
		len--;
	else{
		if((p = runestrrchr(b, L'/')) == nil){
			fprint(2, "'%S' garbled url\n", b);
			return runestrdup(L"");
		}
		len = p-b;
	}
	return runesmprint("%.*S/%S", len, b, u);
}

char *
urlsuffix(char *u)
{
	char *p;
	if (p = strrchr(u, '?'))
		*p = '\0';
	else p = u;
	if (p = strrchr(p, '.'))
		p++;
	else p = u;
	return p;
}

char *
mimetypefixup(char *q)
{
	char *p;
	if (p = strchr(q, ';'))
		*p = '\0';
	if (p = strchr(q, ' '))
		*p = '\0';
	return q;
}

Rune *
geturl(Rune *base, Rune *s)
{
	return dotdot(urlcombine(base, javascript(s)));
}
