#include <u.h>
#include <libc.h>
#include <draw.h>
#include <html.h>
#include "abaco.h"

static Webimage *images;

static void
loadimg(Webimage *i)
{
	Image *im, *tim;
	Rune *src;
	char url[256];
	char *type, *p;
	int conn, fd;

	if (erfork(RFPROC|RFMEM|RFFDG))
		return;

	src = urlcombine(docinfo->base, i->imsrc);
	fd = getwebfd(src, &conn);
	type = getwebattr(conn, "contenttype");

	snprint(url, sizeof(url), "%S", src);
	mimetypefixup(url);

	p = urlsuffix(url);

	if (!cistrcmp(type, "image/jpeg") ||
		!cistrcmp(type, "image/pjpeg") ||
		!cistrcmp(p, "jpeg") || !cistrcmp(p, "jpg"))
		fd = filter(fd, "/bin/jpg", "jpg", "-9t");
	else if (!cistrcmp(type, "image/gif") || !cistrcmp(p, "gif"))
		fd = filter(fd, "/bin/gif", "gif", "-9t");
	else if (!cistrcmp(type, "image/png") || !cistrcmp(p, "png"))
		fd = filter(fd, "/bin/png", "png", "-9t");
	else {
		fprint(2, "unknown image type: %s, suffix %s\n", type, p);
		close(fd);
		exits(0);
	}

	if (im = readimage(display, fd, 1)) {
		lockdisplay(display);
		tim = eallocimage(display, im->r, RGB24, 1, -1);
		draw(tim, tim->r, im, nil, ZP);
		freeimage(im);
		i->i = tim;
		unlockdisplay(display);
	}

	free(src);
	close(fd);
	exits(0);
}

static Webimage *
findimg(Webimage *i, Rune *src)
{
	while(i){
		if (!runestrcmp(src, i->imsrc))
			break;
		i = i->next;
	}
	return i;
}

static void
loadimages(Docinfo *di)
{
	Iimage *ip, *tip;
	Webimage *wi;
	Waitmsg *wm;
	int conc;

	conc = 0;
	for (ip = di->images; ip; ip = ip->nextimage) {
		if (ip->imsrc == nil)
			continue;
		if (findimg(images, ip->imsrc))
			continue;

		if (++conc > Nparallel && (wm = wait()) != nil) {
			write(refresh, "i", 1);
			free(wm);
			conc--;
		}

		wi = emalloc(sizeof(Webimage));
		wi->next = images;
		images = wi;
		wi->imsrc = ip->imsrc;
		ip->aux = wi;
		loadimg(wi);
		for(tip=ip->nextimage; tip; tip=tip->nextimage)
			if(!runestrcmp(tip->imsrc, ip->imsrc))
				tip->aux = wi;
	}

	while(wm = wait()){
		write(refresh, "i", 1);
		free(wm);
	}
}

void
loadpage(Rune *srcu)
{
	uchar *buf;
	char *actstr;
	Rune *actrune;
	long size = 1024, i , n;
	int conn, fd;
	Anchor *a;

	fd = getwebfd(srcu, &conn);
	if (erfork(RFPROC|RFMEM|RFFDG))
		return;

	buf = emalloc(size);


	actstr = getwebattr(conn, "parsed/url");
	i = 0;
	while((n=read(fd, buf + i, size - i)) > 0){
		i += n;
		if (i == size) {
			size *= 2;
			buf = erealloc(buf, size);
		}
	}
	buf = erealloc(buf, i+1);
	buf[i] = 0;
	n = i;
	close(fd);

	actrune = runesmprint("%s", actstr);
	free(Url);
	Url = actrune;

	docitems = parsehtml(buf, n, actrune, TextHtml, charset(buf), &docinfo);
	USED(docitems);
	free(buf);

	for(a=docinfo->anchors; a; a=a->next)
		addanchor(a);

	sortanchors();
	write(refresh, "i", 1);
	loadimages(docinfo);
	write(refresh, "f", 1);
	exits(0);
}
