#include <u.h>
#include <libc.h>
#include <draw.h>
#include <html.h>
#include "abaco.h"

static void renderitem(Item *);

static Image *dbuf;

static
void
rendercheck(Iformfield *i, int isradio)
{
	drawcheck(dbuf, i->r, isradio, i->formfield->flags&FFchecked);
	addselect(i->r, (Item *)i);
	i->aux = dbuf;
}

static
void
renderbutton(Iformfield *i, int down)
{
	drawbutton(dbuf, i->r, i->formfield->value, down);
	addselect(i->r, (Item *)i);
}

static
void
renderrule(Irule *i)
{
	Point p0, p1;

	p0 = addpt(i->r.min, Pt(0, Space));
	p1 = p0;
	p1.x += Dx(i->r);
	line(dbuf, p0, p1, 0, 0, i->size-2, display->black, ZP);
}

static
void
renderimage(Iimage *i)
{
	Rectangle tr;
	Point clipshift, picoff;
	Webimage *wi;
	Image *pic;

	if (i->aux == nil)
		return;

	wi = (Webimage *)i->aux;

	if((pic = wi->i) == nil)
		return;

	tr= i->r;
	if (tr.max.x == Dx(dbuf->r))
		tr.max.x = dbuf->clipr.max.x;

	clipshift = tr.min;
	if (rectclip(&tr, dbuf->clipr)){
		clipshift = subpt(tr.min, clipshift);
		picoff = addpt(pic->r.min, clipshift);
		rectclip(&tr, rectaddpt(pic->r, tr.min));
		draw(dbuf, tr, pic, nil, picoff);
		if (i->anchorid)
			addselect(tr, i);
	}
}

static
void
rendertextfield(Iformfield *i)
{
	Rectangle r;
	Font *f;
	Rune *rs;

	f = getfont(WFont);
	rs = i->formfield->value;
	drawtextarea(dbuf, i->r, rs);
	r = insetrect(i->r, Butmargin);
	addsnarf(r, rs, rs ? runestrlen(rs) : 0, f, display->black, ULnone);
	addselect(r, (Item *)i);
	if(i->aux)
		free(i->aux);
	i->aux = mktextfield(i->formfield, dbuf, r);
}

static
void
renderformfield(Iformfield *i)
{
	switch(i->formfield->ftype) {
	case Ftext:
	case Ftextarea:
		rendertextfield(i);
		break;
	case Fpassword:
		 rendertextfield(i);
		break;
	case Fcheckbox:
		rendercheck(i, 0);
		break;
	case Fradio:
		rendercheck(i, 1);
		break;
	case Fbutton:
	case Fsubmit:
		renderbutton(i, 0);
		break;
	case Fselect:
	case Fhidden:
	case Fimage:
	case Freset:
	case Ffile:
		break;
	}
}

static
void
rendertable(Itable *i)
{
	Tablecell *c;

	for(c=i->table->cells; c; c=c->next)
		renderitem(c->content);
}

static
void
renderfloat(Ifloat *i)
{
	renderitem(i->item);
}

static
void
renderitem(Item *items)
{
	Item *i;

	for(i=items; i; i=i->next){
		switch(i->tag){
		case Itexttag:
			break;
		case Iruletag:
			renderrule((Irule *)i);
			break;
		case Iimagetag:
			renderimage((Iimage *)i);
			break;
		case Iformfieldtag:
			renderformfield((Iformfield *)i);
			break;
		case Itabletag:
			rendertable((Itable *)i);
			break;
		case Ifloattag:
			renderfloat((Ifloat *)i);
			break;
		case Ispacertag:
			break;
		default:
			fprint(2, "unknown item tag %d\n", i->tag);
			break;
		}
	}
}

static
Image *
getbg(Docinfo *di)
{
	Image *bg;

	if(di->backgrounditem){
		if(di->backgrounditem->aux)
			bg = ((Webimage *)di->backgrounditem->aux)->i;
		else
			bg = display->white;
	}
	else
		bg = getcolor(di->background.color);

	return bg;
}

Image *
render(Rectangle r, Docinfo *di, Item *i)
{
	if(!i)
		return nil;

	freesnarf();
	freeselect();
	dbuf = eallocimage(display, Rpt(Pt(0,0),getsize(r,i)), 
		screen->chan, 0, DNofill);
	draw(dbuf, dbuf->r, getbg(di), nil, ZP);
	drawtext(dbuf);
	renderitem(i);

	return dbuf;
}
