#include <u.h>
#include <libc.h>
#include <draw.h>
#include <html.h>
#include "abaco.h"

typedef struct Rectl Rectl;


struct Rectl
{
	Rectl *next;
	Rectangle r;
	Item *i;
	struct{
		int min;
		int max;
		int wall;
	}x;
};

static Rectl *sizeitem(Item *i, Rectl *r);

#define ispre(i) (!(i->state & IFwrap))
#define xsize(x) (x.wall - x.min)

static
int
lineheight(Rectl *rl)
{
	Rectl *l;
	int h;

	h = Dy(rl->r);
	for(l=rl; l; l=l->next)
		if(h < Dy(l->r))
			h = Dy(l->r);
	return h;
}

static
Rectl *
pop(Rectl *rl)
{
	Rectl *l;

	l = rl->next;
	l->x = rl->x;
	free(rl);
	return l;
}

static
Rectl *
push(Rectl *rl, Rectangle r)
{
	Rectl *l;

	l = emalloc(sizeof(Rectl));
	l->r = r;
	l->x = rl->x;
	l->next = rl;
	return l;
}

static
void
justify(Rectl *rl)
{
	Rectl *l;
	int x, state, indent;

	if(!rl)
		return;

	if(!rl->i){
		if(rl->next && rl->next->i)
			state = rl->next->i->state;
		else
			return;
	}else
		state = rl->i->state;

	indent = ((state&IFindentmask)>>IFindentshift) * Tabspace;
	x = rl->x.min;
	if(!(state&IFrjust) && !(state&IFcjust) && !indent)
		return;

	if(ispre(rl->i))
		return;

	if(state & IFrjust)
		x += xsize(rl->x) - rl->r.max.x-x;
	else if(state & IFcjust)
		x += xsize(rl->x)/2 - (rl->r.max.x-x)/2 ;

	x += indent;

	for(l=rl; l; l=l->next){
		if(!l->i)
			continue;
		l->i->r = rectaddpt(l->i->r, Pt(x, 0));
	}
	justifytext(x,rl->r.min.y);
	justifyselect(x, rl->r.min.y);
}

static
Rectl *
newline(Rectl *rl, int space)
{
	int h;

	h = lineheight(rl);
	justify(rl);
	while(rl->next)
		rl = pop(rl);

	rl->r.min.y += h+Space;
	if(space)
		rl->r.min.y += Space;

	rl->r.max = addpt(rl->r.min, Pt(0, Space*2));

	return rl;
}

static
Rectl *
getrectl(Item *i, Rectl *rl, int w, int h, int force)
{
	Rectangle r;
	int indent;

	r = rl->r;
	r = Rect(r.max.x, r.min.y, r.max.x+w, r.min.y+h);
	indent = ((i->state&IFindentmask)>>IFindentshift) * Tabspace;
	if(r.max.x>=rl->x.wall-indent && !force){
		rl =  newline(rl, 0);
		rl->r.max = addpt(rl->r.min, Pt(w,h));
		goto Ret;
	}
	rl = push(rl, r);

	if(force && r.max.x>rl->x.max)
		rl->x.max = r.max.x;
Ret:
	if(i->tag != Itexttag)
		i->r = rl->r;

	rl->i = i;
	
	return rl;	
}

static
Rectl *
sizebutton(Rectl *rl, Iformfield *i)
{
	Font *f;
	int w, h;

	f = getfont(WFont);
	w = runestringwidth(f, i->formfield->value) + 2*Butmargin;
	h = f->height + 2*Butmargin;

	return getrectl(i, rl, w, h, ispre(i));
}

static
Rectl *
sizerule(Irule *i, Rectl *rl)
{
	rl = newline(rl, 0);
	rl = getrectl(i, rl, xsize(rl->x), Space+i->size-2+Space, 1);
	rl = newline(rl, 0);

	return rl;
}

static
Rectl *
sizeimage(Iimage *i, Rectl *rl)
{
	Webimage *wi;
	Image *pic;

	if (i->aux == nil)
		return rl;

	wi = (Webimage *)i->aux;

	if((pic = wi->i) == nil)
		return rl;

	if(!ispre(i) && Dx(pic->r)+rl->r.max.x > xsize(rl->x))
		rl = newline(rl, 0);

	return getrectl(i, rl, Dx(pic->r), Dy(pic->r), ispre(i) || Dx(pic->r)>xsize(rl->x));
}

static
Rectl *
sizetextfield(Iformfield *i, Rectl *rl)
{
	Font *f;
	int w, h;

	f = getfont(WFont);
	w = runestringwidth(f, L"0") * i->formfield->size;
	w  +=  2*Butmargin;
	h = f->height + 2*Butmargin;

	return getrectl(i, rl, w, h, ispre(i));
}

static
Rectl *
sizeformfield(Iformfield *i, Rectl *rl)
{
	switch(i->formfield->ftype) {
	case Ftext:
	case Ftextarea:
		rl = sizetextfield(i, rl);
		break;
	case Fpassword:
		rl = sizetextfield(i, rl);
		break;
	case Fcheckbox:
	case Fradio:
		rl = getrectl(i, rl, Checkwidth+Space, Checkheight, ispre(i));
		break;
	case Fbutton:
	case Fsubmit:
		rl = sizebutton(rl, i);
		break;
	case Fhidden:
	case Fimage:
	case Freset:
	case Ffile:
	case Fselect:
		break;
	}
	return rl;
}

static
Rectl *
sizetable(Itable *i, Rectl *rl)
{
	Tablecell *c;

	for(c=i->table->cells; c; c=c->next)
		rl = sizeitem(c->content, rl);

	return rl;
}

static
Rectl *
sizefloat(Ifloat *i, Rectl *rl)
{
	return sizeitem(i->item, rl);
}

static
Rectl *
sizespacer(Ispacer *i, Rectl *rl)
{
	if(i->spkind != ISPnull)
		return  getrectl(i, rl, Space, Space, ispre(i));
	else
		return rl;
}

static
Rectl *
sizetext(Itext *i, Rectl *rl)
{
	Image *fc;
	Font *f;
	Rune *s;
	int w, n, x, aux;

	f = getfont(i->fnt);
	fc = getcolor(i->fg);
	x = runestringwidth(f, L" ");
	if(ispre(i)){
		rl = getrectl(i, rl, runestringwidth(f, i->s), f->height, 1);
		addsnarf(rl->r, i->s, runestrlen(i->s), f, fc, i->ul);
		if(i->anchorid)
			addselect(rl->r, i);
		return rl;
	}
	for(s=i->s; *s; s+=n){
		n = 0;
		while(isspacerune(*s))
			s++;
		if(!*s)
			break;
		while(s[n] && !isspacerune(s[n]))
			n++;
		w = runestringnwidth(f, s, n);
		aux = 0;
		if(!s[n]){
			s[n++] = L' ';
			aux = 1;
		}else if (isspacerune(s[n]))
			n++;

		w += x;
		rl = getrectl(i, rl, w, f->height, 0);

		if(aux)
			s[--n] = 0;

		addsnarf(rl->r, s, n, f, fc, i->ul);
		if(i->anchorid)
			addselect(rl->r, i);
	}

	return rl;
}

static
Rectl *
sizeitem(Item *items, Rectl *rl)
{
	Item *i;

	for(i=items; i; i=i->next){
		if (i->state & IFbrk)
			rl = newline(rl, 0);
		if (i->state & IFbrksp)
			rl = newline(rl, 1);

		switch (i->tag){
		case Itexttag:
			rl = sizetext((Itext *)i, rl);
			break;
		case Iruletag:
			rl = sizerule((Irule *)i, rl);
			break;
		case Iimagetag:
			rl = sizeimage((Iimage *)i, rl);
			break;
		case Iformfieldtag:
			rl = sizeformfield((Iformfield *)i, rl);
			break;
		case Itabletag:
			rl = sizetable((Itable *)i, rl);
			break;
		case Ifloattag:
			rl = sizefloat((Ifloat *)i, rl);
			break;
		case Ispacertag:
			rl = sizespacer((Ispacer *)i, rl);
			break;
		default:
			fprint(2, "unknown item tag %d\n", i->tag);
			break;
		}
	}
	return rl;
}

static
Point
getsize0(Rectangle r, Item *i)
{
	Rectl *rl;
	int x, y;


	rl = emalloc(sizeof(Rectl));
	rl->x.min = r.min.x;
	rl->x.max = rl->x.wall = r.max.x;
	rl = sizeitem(i, rl);
	rl = newline(rl, 0);
	y = rl->r.max.y-Space-r.min.y;
	x = rl->x.max - rl->x.min;

	if(y < Dy(r))
		y = Dy(r);

	while(rl->next)
		rl = pop(rl);
	free(rl);

	return Pt(x,y);
}

Point
getsize(Rectangle r, Item *i)
{
	return getsize0(Rect(0,0,Dx(r),Dy(r)), i);
}
