#include <u.h>
#include <libc.h>
#include <draw.h>
#include <html.h>
#include "abaco.h"

typedef struct Snarf Snarf;

struct Snarf
{
	Rectangle r;
	Rune *s;
	int n;
	int u;
	Font *f;
	Image *c;
	Snarf *next;
};

static Snarf *list;

void
addsnarf(Rectangle r, Rune *rs, int n, Font *f, Image *c, int u)
{
	Snarf *s, *t;

	s = emalloc(sizeof(Snarf));
	s->r = r;
	s->s = rs;
	s->n =n;
	s->f = f;
	s->c = c;
	s->u = u;
	if(list == nil)
		list = s;
	else{
		t = list;
		while(t->next)
			t = t->next;
		t->next = s;
	}
}

void
freesnarf(void)
{
	Snarf *s;

	while(list){
		s = list->next;
		free(list);
		list = s;
	}
}

static Image *select = nil;

void
drawsnarf(Image *b, Point top, Point bot, Point sp)
{
	Snarf *s;
	Rectangle r;
	int snarfing = 0;

	if(select == nil)
		select = eallocimage(display, Rect(0,0,1,1), screen->chan, 1, 0xCCCCCCFF);

	for(s=list; s; s=s->next){
		r = s->r;
		if(ptinrect(top, r) || (r.min.y>top.y && r.max.y<bot.y))
			snarfing = 1;
		if(ptinrect(bot, r) || r.min.y > bot.y)
			snarfing = 0;
		if(snarfing){
			r = rectaddpt(r, sp);
			draw(b, r, select, nil, ZP);
			drawstringn(b, r.min, s->c, s->f, s->s, s->n, s->u);
		}
	}
}


Rune *
dosnarf(Point top, Point bot)
{
	Snarf *s;
	Rune *rs = nil;
	int snarflen, snarfalloc, snarfing, i, y;

	snarflen = snarfalloc = snarfing = 0;

	y = list->r.min.y;
	for(s=list; s; s=s->next){
		if(ptinrect(top, s->r) || (s->r.min.y>top.y && s->r.max.y<bot.y))
			snarfing = 1;
		if(ptinrect(bot, s->r) || s->r.min.y > bot.y)
			snarfing = 0;

		if(snarfing){
			if(snarflen+s->n+2 >= snarfalloc){
				snarfalloc = snarflen+s->n+2;
				rs = erealloc(rs, snarfalloc * sizeof(Rune));
			}
			for(i=0; i<s->n; i++)
				rs[snarflen++] = s->s[i];

			if(s->r.min.y > y){
				y = s->r.min.y;
				rs[snarflen++] = L'\n';
			}else
				rs[snarflen++] = L' ';

			rs[snarflen] = 0;
		}

	}
	return rs;
}

void
drawtext(Image *b)
{
	Snarf *s;

	for(s=list; s; s=s->next)
		drawstringn(b, s->r.min, s->c, s->f, s->s, s->n, s->u);
}

void
justifytext(int x, int y)
{
	Snarf *s;

	for(s=list; s; s=s->next)
		if(s->r.min.y == y)
			s->r = rectaddpt(s->r, Pt(x,0));
}
