#include <u.h>
#include <libc.h>
#include <draw.h>
#include <html.h>
#include <plumb.h>

extern int plumbfd;
static char *Wmtpt = "/mnt/web";

void*
emalloc(ulong n)
{
	void *p;

	p = malloc(n);
	if(p == nil)
		sysfatal("malloc failed: %r");
	setmalloctag(p, getcallerpc(&n));
	memset(p, 0, n);
	return p;
}

void*
erealloc(void *p, ulong n)
{
	p = realloc(p, n);
	if(p == nil)
		sysfatal("realloc failed: %r");
	setmalloctag(p, getcallerpc(&n));
	return p;
}

void*
estrdup(char *s)
{
	void *p;

	p = strdup(s);
	if(p == nil)
		sysfatal("strdup failed: %r");
	setmalloctag(p, getcallerpc(&s));
	return p;
}

Rune*
erunestrdup(Rune *r)
{
	void *p;

	p = runestrdup(r);
	if(p == nil)
		sysfatal("runestrdup failed: %r");
	setmalloctag(p, getcallerpc(&r));
	return p;
}

int
erfork(int fl)
{
	int r;
	if ((r = rfork(fl)) < 0)
		sysfatal("rfork: %r");
	return r;
}

int
getwebfd(Rune *url, int *con)
{
	int cfd, fd, n;
	char buf[256];

	snprint(buf, sizeof(buf), "%s/clone", Wmtpt);
	if ((cfd = open(buf, ORDWR)) < 0)
		sysfatal("open %s: %r", buf);

	if ((n = read(cfd, buf, 1024)) <= 0)
		sysfatal("reading clone: %r");
	buf[n] = '\0';
	*con = atoi(buf);

	fprint(cfd, "url %S", url);

	snprint(buf, sizeof(buf), "%s/%d/body", Wmtpt, *con);
 	if ((fd = open(buf, OREAD)) < 0) {
		close(cfd);
		sysfatal("open %S: %r", url);
	}
	close(cfd);
	return fd;
}

char *
getwebattr(int con, char *name)
{
	int n, fd;
	char buf[1024];

	snprint(buf, sizeof(buf), "%s/%d/%s", Wmtpt, con, name);
	if ((fd = open(buf, OREAD)) < 0)
		sysfatal("%s - cannot open: %r", buf);
	if ((n = read(fd, buf, sizeof(buf)-1)) < 0)
		sysfatal("%s - cannot read: %r", buf);
	close(fd);
	buf[n] = '\0';
	return estrdup(buf);
}

ulong
runehash(Rune *s)
{
	ulong x = 0;
	do
		x = ((x>>25) + (x<<7)) ^ *s;
	while (*s++);
	return x;
}

int
filter(int fd, char *prog, char *arg0, char *arg1)
{
	int p[2];
	pipe(p);

	if(!erfork(RFFDG|RFREND|RFNOWAIT|RFPROC|RFNOMNT)) {
		close(p[0]);
		dup(p[1], 1);
		dup(fd, 0);
		execl(prog, arg0, arg1, nil);
		sysfatal("exec failed");
	}
	close(fd);
	close(p[1]);

	return p[0];
}

void
plumbrunestring(Rune *str)
{
	Plumbmsg pmsg;

	pmsg.src = "abaco";
	pmsg.dst = nil;
	pmsg.wdir = "/tmp";
	pmsg.type = "text";
	pmsg.attr = nil;
	pmsg.data = smprint("%S", str);
	pmsg.ndata = -1;
	plumbsend(plumbfd, &pmsg);
	free(pmsg.data);
}

void
snarfrunestring(Rune *str)
{
	char *p;
	int fd;
	p = smprint("%S", str);
	fd = open("/dev/snarf", OWRITE);
	write(fd, p, strlen(p));
	close(fd);
	free(p);
}

void
newwin(void)
{
	char spec[100], srv[100];
	int srvfd, cons, pid;

	if(erfork(RFFDG|RFPROC|RFNAMEG|RFENVG|RFNOTEG|RFNOWAIT))
		exits(0);

	srvfd = open("/env/wsys", OREAD);
	read(srvfd, srv, sizeof(srv));
	close(srvfd);

	pid = getpid();
	srvfd = open(srv, ORDWR);
	if(srvfd == -1)
		sysfatal("can't open %s: %r", srv);

	sprint(spec, "new -pid %d", pid);
	if(mount(srvfd, -1, "/mnt/wsys", 0, spec) == -1)
		sysfatal("can't mount /mnt/wsys: %r (spec=%s)", spec);

	close(srvfd);
	unmount("/mnt/acme", "/dev");
	bind("/mnt/wsys", "/dev", MBEFORE);
	cons = open("/dev/cons", OREAD);
	if(cons==-1){
	NoCons:
		sysfatal("can't open /dev/cons: %r");
	}
	dup(cons, 0);
	close(cons);
	cons = open("/dev/cons", OWRITE);
	if(cons==-1)
		goto NoCons;
	dup(cons, 1);
	dup(cons, 2);
	close(cons);
}

void
download(Rune *src)
{
	Rune rs[1024];
	char url[1024], file[1024];

	if(!erfork(RFPROC|RFFDG|RFNOTEG)){
		runesnprint(rs, sizeof(rs), "%S", src);
		snprint(url, sizeof(url), "%S", rs);
		snprint(file, sizeof(file), "/tmp/%S", runestrrchr(rs, L'/')+1);
		if(file[5] == 0)
			sprint(file, "/tmp/index.html");
		execl("/bin/hget","hget", "-o", file, url, nil);
		sysfatal("exec failed");
	}
}
