#include <u.h>
#include <libc.h>
#include <draw.h>
#include <html.h>
#include <keyboard.h>
#include "abaco.h"

typedef struct Textfield Textfield;

struct Textfield
{
	Formfield *f;
	int pos;
	int len;
	int alloc;
	Rectangle r;
	Image *i;
};

static
void
putchar(Textfield *t, Rune c)
{
	t->len++;
	if (t->len >= t->alloc) {
		t->alloc *= 2;
		t->f->value = erealloc(t->f->value, t->alloc * sizeof(Rune));
	}
	t->f->value[t->len-1] = c;
	t->f->value[t->len] = 0;
}

void *
mktextfield(Formfield *f, Image *i, Rectangle r)
{
	Textfield *t;

	t = emalloc(sizeof(Textfield));
	t->f = f;
	t->r = r;
	t->i = i;
	if (f->value) {
		t->len = runestrlen(f->value);
		t->alloc = t->len+1;
	} else {
		t->len =0;
		t->alloc = 16;
		t->f->value = emalloc(t->alloc * sizeof(Rune));
		t->f->value[t->len] = 0;
	}
	return (void *)t;
}

static
void
submitform(Formfield *tff, int subfl)
{
	Formfield *ff = tff;
	Rune *x, *y, *z;
	Rune sep = L'?';
	x = runesmprint("%S", ff->form->action);
	for(ff=ff->form->fields; ff; ff=ff->next) {
		if(ff->value==nil || (ff->ftype==Fsubmit && (!subfl || ff != tff)))
			continue;
		if((ff->ftype==Fradio || ff->ftype==Fcheckbox) && !(ff->flags&FFchecked))
			continue;
		z = runestrdup(ff->value);
		y = runesmprint("%S%C%S=%S", x, sep, ff->name, urlencode(z));
		sep = L'&';
		free(x);
		free(z);
		x = y;
	}

	y = geturl(location, x);
	fprint(2, "xxx: %S\n", y);
	free(x);
	plumbrunestring(y);
	free(y);
}

static
void
keyboard(Iformfield *f, Rune r)
{
	Textfield *t;

	t = (Textfield *)f->aux;
	switch(r){
	case '\b':
		if (!t->len)
			return;
		t->len--;
		t->f->value[t->len] = 0;
		break;
	case '\n':
		submitform(f->formfield, 0);
		break;
	default:
		putchar(t, r);
		break;
	}
}

static
void
drawtextfield(Iformfield *f)
{
	Textfield *t;
	Rune *s;
	int i, n;

	t = (Textfield *)f->aux;

	draw(t->i, t->r, display->white, nil, ZP);
	s = runestrdup(t->f->value);
	if(!s)
		return;

	n = runestrlen(s);
	if(t->f->ftype == Fpassword){
		for(i=0; i<n; i++)
			s[i] = L'*';
	}
	if(n > t->f->size)
		n = t->f->size;

	runestringn(t->i, t->r.min, display->black, ZP, getfont(WFont), s, n);
	free(s);
}

typedef struct Select Select;

struct Select
{
	Select *next;
	Rectangle r;
	Item *i;
};

static Select *list;

void
addselect(Rectangle r, Item *item)
{
	Select *s;
	s = emalloc(sizeof(Select));
	s->r = r;
	s->i = item;
	s->next = list;
	list = s;
}

void
freeselect(void)
{
	Select *s;

	while(list){
		s = list->next;
		free(list);
		list = s;
	}
}

void
justifyselect(int x, int y)
{
	Select *s;

	for(s=list; s; s=s->next)
		if(s->r.min.y == y)
			s->r = rectaddpt(s->r, Pt(x,0));
}

Rune *
findhref(Point p)
{
	Select *s;

	for(s=list; s; s=s->next)
		if(ptinrect(p, s->r))
			if(s->i->anchorid)
				return anchor(s->i->anchorid)->href;
	return nil;
}

int
kbdevent(Point p, Rune r)
{
	Select *s;
	Iformfield *ff;
	int type;

	for(s=list; s; s=s->next)
		if(ptinrect(p, s->r))
			if(s->i->tag == Iformfieldtag){
				ff = (Iformfield *)s->i;
				type = ff->formfield->ftype;
				if(type==Ftext || type==Fpassword){
					keyboard(ff, r);
					drawtextfield(ff);
					return 1;
				}
			}
	
	return 0;
}

static
void
doradio(Formfield *f)
{
	Formfield *ff;
	Select *s;
	Iformfield *i = nil;

	USED(i);
	for(ff= f->form->fields; ff; ff=ff->next){
		if(ff == f)
			continue;
		if(!runestrcmp(ff->name, f->name)){
			ff->flags &= ~FFchecked;
			for(s=list; s; s=s->next)
				if(s->i->tag == Iformfieldtag){
					i = (Iformfield *)s->i;
					if(i->formfield == ff)
						drawcheck(i->aux, i->r, 1, 0);
				}
		}
	}
}

static
void
docheck(Iformfield *i)
{
	Formfield *ff;
	Image *b;

	ff = i->formfield;

	if(ff->flags & FFchecked)
		ff->flags &= ~FFchecked;
	else
		ff->flags |= FFchecked;

	b = i->aux;
	if(ff->ftype == Fradio)
		doradio(ff);
				
	drawcheck(b, i->r, ff->ftype==Fradio, ff->flags&FFchecked);
}

static
int
formevent(Iformfield *i)
{
	Formfield *ff;
	int ans = 0;

	ff = (Formfield *)i->formfield;
	switch(ff->ftype){
	case Fsubmit:
		submitform(ff, 1);
		break;
	case Fradio:
	case Fcheckbox:
		docheck(i);
		ans = 1;
		break;
	}

	return ans;
}

int
mouseevent(Point p)
{
	Select *s;

	for(s=list; s; s=s->next)
		if(ptinrect(p, s->r)){
			if(s->i->tag == Iformfieldtag)
				return formevent((Iformfield *)s->i);
			else
				break;
		}

	return 0;
}
